#ifndef RAT_H_
#define RAT_H_

#include <iosfwd>

/**
 * Rational number class
 *
 * Invariant: p and q are relative primes
 */
class rat {
  int p;
  int q;

public:
  rat(int p_, int q_ = 1) : p(p_), q(q_)
  { simplify(); }

private:
  void simplify();

public:
  int get_p() const { return p; }
  int get_q() const { return q; }

  void add(const rat& o);

  rat& operator+=(const rat& o) {
    add(o);
    return *this;
  }

};

inline bool operator==(
    const rat& r1, const rat& r2) {
  return r1.get_p() == r2.get_p()
      && r1.get_q() == r2.get_q();
}

std::ostream& operator<<(
    std::ostream& o, const rat& r);

#endif // RAT_H_
